/*
 * Copyright (C) 2020 - 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <stdlib.h>
#include <string.h>

#include "atari8b.h"


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_hrpl_get_mime_types()
{
    static const char *mime_types[] = {
        "image/x-hires-player-missile",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_hrpl_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    h->info = info;
    h->info->threadsafe = 1;
    h->surface = NULL;
    return h;
}


static void
_hrpl_free(abydos_plugin_handle_t *h)
{
    if(h->surface)
        cairo_surface_destroy(h->surface);
    free(h);
}


static int
_hrpl_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    uint8_t *bitplane[2];
    atari8b_cairo_color_t palette[4];
    uint8_t color[3];
    if(len != 19203)
        return -1;
    h->info->width = 320;
    h->info->height = 240;
    bitplane[0] = (uint8_t*)data;
    bitplane[1] = (uint8_t*)data+9600;
    memcpy(color,data+19200,3);
    palette[0] = atari8b_palette[color[0]];
    palette[1] = atari8b_palette[color[1]];
    palette[2] = atari8b_palette[color[1]|color[2]];
    palette[3] = atari8b_palette[color[2]];
    h->surface = atari8b_decode_image(bitplane, palette);
    return 0;
}


static cairo_surface_t *
_hrpl_get_image_surface(abydos_plugin_handle_t *h,
                       int page,
                       cairo_bool_t *layer_visibility,
                       int variant,
                       double time,
                       int frame)
{
    return cairo_surface_reference(h->surface);
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _hrpl_get_mime_types,
        .new = _hrpl_new,
        .free = _hrpl_free,
        .create_from_data = _hrpl_create_from_data,
        .get_image_surface = _hrpl_get_image_surface
    };
    return &p;
}
