/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <debug.h>
#include <math.h>
#include <string.h>
#include <sys/types.h>                  /* ssize_t */

#include "cgmparse.h"
#include "cgmread-binary.h"
#include "cgmread-text.h"
#include "elements.h"


int handle_class0(cgm_context_t*, int, cgm_read_t*);
int handle_class1(cgm_context_t*, int, cgm_read_t*);
int handle_class2(cgm_context_t*, int, cgm_read_t*);
int handle_class3(cgm_context_t*, int, cgm_read_t*);
int handle_class4(cgm_context_t*, int, cgm_read_t*);
int handle_class5(cgm_context_t*, int, cgm_read_t*);
int handle_class6(cgm_context_t*, int, cgm_read_t*);
int handle_class7(cgm_context_t*, int, cgm_read_t*);
int handle_class8(cgm_context_t*, int, cgm_read_t*);
int handle_class9(cgm_context_t*, int, cgm_read_t*);
int handle_class15(cgm_context_t*, int, cgm_read_t*);


static void
_context_init(cgm_context_t *ctx, const cgm_interface_t *vt, void *userdata)
{
    memset(ctx, 0, sizeof(cgm_context_t));
    ctx->state = METAFILE_CLOSED;
    ctx->vt = vt;
    ctx->userdata = userdata;
    cgm_element_init(&ctx->pElement);
    //pElement.reset( new CGMElements );
    //pCopyOfE.reset( new CGMElements );
    loa_buffer_init(&ctx->vb, 0, 0, 0, 0);
    //nile_init(&state.ni);
}


static void
_context_done(cgm_context_t *ctx)
{
    cgm_element_done(&ctx->pElement);
    loa_buffer_done(&ctx->vb);
}


static int
_on_entry(void *userdata, const cgm_element_t *e, cgm_read_t *r){
    cgm_context_t *ctx = userdata;
    //CGM *cgm = (CGM*)userdata;
    cgm_class_handler_t handler;
    //cgm->read = r;

    //DEBUG_p("class: %d, id: 0x%02x", e->klass, e->id);
    switch(e->klass){
    case 0:
        handler = handle_class0;
        break;
    case 1:
        handler = handle_class1;
        break;
    case 2:
        handler = handle_class2;
        break;
    case 3:
        handler = handle_class3;
        break;
    case 4 :
        handler = handle_class4;
        break;
    case 5:
        handler = handle_class5;
        break;
    case 6:
        handler = handle_class6;
        break;
    case 7:
        handler = handle_class7;
        break;
    case 8:
        handler = handle_class8;
        break;
    case 9:
        handler = handle_class9;
        break;
    case 15:
        handler = handle_class15;
        break;
    default:
        DEBUG_p("invalid class: %d", e->klass);
        return -1;
    }

    if(handler(ctx, e->id, r) < 0){
        DEBUG_p("parsing %02x of class %d failed", e->id, e->klass);
        return -1;
    }

    if(4 == e->klass)
        ctx->mnAct4PostReset = 0;

    //cgm->mnActCount++;
    return 0;
}

int
cgm_interprete(const uint8_t *data, size_t len, const cgm_interface_t *vt, void *userdata)
{
    int ret;
    cgm_context_t ctx;
    ctx.vt = vt;
    ctx.userdata = userdata;

    if(len<4)
        return -1;
    if(1 == be16toh(*(uint16_t*)data) >> 5){
        DEBUG_p("binary file");
        _context_init(&ctx, vt, userdata);
        ret = cgm_read_binary(data, len, _on_entry, &ctx);
        _context_done(&ctx);
    }else if(!strncmp((char*)data, "BegM", 4)){
        DEBUG_p("text file");
        _context_init(&ctx, vt, userdata);
        ret = cgm_read_text(data, len, _on_entry, &ctx);
        _context_done(&ctx);
    }else{
        DEBUG_p("incorrect file");
        return -1;
    }
    return ret;
}


int cgm_set_map_mode(cgm_context_t *ctx, cgm_read_t *read)
{
    int nAngReverse = 1;
    ctx->mnVDCdx = ctx->pElement.aVDCExtent.size.x;
    ctx->mnVDCXadd = -ctx->pElement.aVDCExtent.pos.x;
    ctx->mnVDCXmul = 1;
    if(ctx->mnVDCdx < 0){
        nAngReverse ^= 1;
        ctx->mnVDCdx *= -1;
        ctx->mnVDCXmul = -1;
    }

    ctx->mnVDCdy = ctx->pElement.aVDCExtent.size.y;
    ctx->mnVDCYadd = -ctx->pElement.aVDCExtent.pos.y;
    ctx->mnVDCYmul = 1;
    if(ctx->mnVDCdy < 0){
        nAngReverse ^= 1;
        ctx->mnVDCdy *= -1;
        ctx->mnVDCYmul = -1;
    }
    ctx->mbAngReverse = nAngReverse;

    DEBUG_p("unscaled size: %dx%d", (int)ctx->mnVDCdx, (int)ctx->mnVDCdy);

    if(ctx->mnVDCdy == 0.0 || ctx->mnVDCdx == 0.0)
        return -1;

    ctx->width = ctx->mnVDCdx; //  * ctx->mnXFraction;
    ctx->height = ctx->mnVDCdy; // * ctx->mnYFraction;
    DEBUG_p("canvas size: %dx%d", ctx->width, ctx->height);

    return 0;
}


#if 0
void cgm_map_double(cgm_state_t *state, cgm_read_t *r, double *nNumb )
{
    if ( state->pElement.eDeviceViewPortMap != DVPM_FORCED )
        return;

    // point is 1mm * ScalingFactor
    switch ( state->pElement.eDeviceViewPortMode )
    {
        case DVPM_FRACTION :
        {
            *nNumb *= ( state->mnXFraction + state->mnYFraction ) / 2;
        }
        break;

        case DVPM_METRIC :
        {
            // nNumb *= ( 100 * pElement->nDeviceViewPortScale );
            *nNumb *= ( state->mnXFraction + state->mnYFraction ) / 2;
            if ( state->pElement.nDeviceViewPortScale < 0 )
                *nNumb = -*nNumb;
        }
        break;

        case DVPM_DEVICE :
        {

        }
        break;

        default:

            break;
    }
}
#endif


void cgm_default_replacement(cgm_context_t *ctx, cgm_read_t *r)
{
#if 0
    if (maDefRepList.empty())
        return;

    if (state->mbInDefaultReplacement)
    {
        DEBUG_p("filter.icgm" ": " "recursion in ImplDefaultReplacement");
        return;
    }
#endif
/* FIXME!! */
#if 0
    state->mbInDefaultReplacement = true;

    const uint8_t *old_data = read->data;
    const uint8_t *old_end = read->end;

    for ( size_t i = 0, n = maDefRepList.size(); i < n; ++i )
    {
        const uint8_t *data = maDefRepList[ i ].get();
        size_t len = maDefRepSizeList[ i ];
#if 1
        if(cgm_parse(data, len, state->vt, state->userdata) < 0){
            state->mbStatus = false;
            break;
        }
#else
        while(data < end){
            ssize_t bytes = _handle_entry(this, data, end-data);
            if(bytes < 0){
                mbStatus = false;
                break;
            }
            data += bytes;
        }
#endif
    }
    read->data = old_data;
    read->end = old_end;

    state->mbInDefaultReplacement = false;
#endif
}
