/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef cgmparse_h
#define cgmparse_h

#include <stdbool.h>
#include <stddef.h>
#include <stdint.h>

#include "cgmdraw.h"
#include "cgmread.h"
#include "cgmtypes.h"
#include "loa-matrix.h"


typedef enum{
    METAFILE_CLOSED,
    METAFILE_DESCRIPTION,
    PICTURE_DESCRIPTION,
    PICTURE_OPEN,
    PICTURE_CLOSED,

    GLOBAL_SEGMENT_OPEN,
    LOCAL_SEGMENT_OPEN
}cgm_state_t;

typedef struct _cgm_context_t cgm_context_t;

struct _cgm_context_t{
    int width;
    int height;

    cgm_state_t state;

    const cgm_interface_t *vt;
    void *userdata;

    loa_buffer_t vb;
    //nile_t ni;
    CGMElements pElement;

    double              mnVDCXadd;
    double              mnVDCYadd;
    double              mnVDCXmul;
    double              mnVDCYmul;
    bool                mbAngReverse;           // AngularDirection

    double              mnVDCdx;
    double              mnVDCdy;

    bool                mbStatus;
    bool                mbMetaFile;
    bool                mbIsFinished;
    bool                mbPicture;
    bool                mbPictureBody;
    bool                mbFigure;
    bool                mbFirstOutPut;
    bool                mbInDefaultReplacement;
    uint32_t            mnAct4PostReset;
};

typedef int(*cgm_class_handler_t)(cgm_context_t*, int id, cgm_read_t*);

int cgm_interprete(const uint8_t *data, size_t len, const cgm_interface_t*, void *userdata);

int cgm_set_map_mode(cgm_context_t*, cgm_read_t*);
void cgm_map_double(cgm_context_t*, cgm_read_t*, double *nNumb );

#endif /* cgmparse_h */
