/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef cgmread_h
#define cgmread_h

#include <loa-buffer.h>

#include "cgmtypes.h"
#include "loa-matrix.h"

typedef struct _CGMElements CGMElements;
typedef struct _cgm_read_t cgm_read_t;

typedef struct{
    const char *string;
    int klass;
    int id;
}cgm_element_t;

typedef int(*cgm_handler_t)(void *userdata, const cgm_element_t*, cgm_read_t*);

typedef struct{
    int(*eoe)(cgm_read_t*);

    int(*get_I)(cgm_read_t*);
    unsigned int(*get_IX)(cgm_read_t*);
    double(*get_R)(cgm_read_t*);
    const char*(*get_S)(cgm_read_t*, int*);

    unsigned int(*get_color_channel)(cgm_read_t*); /* CCO */
    unsigned int(*get_color_index)(cgm_read_t*); /* CI */
    double(*get_vdc)(cgm_read_t*); /* VDC */
    int(*get_int_precision_bytes)(cgm_read_t*);

    vdc_type_t(*get_vdc_type)(cgm_read_t*);
    ScalingMode(*get_scaling_mode)(cgm_read_t*);
    DeviceViewPortMode(*get_device_view_port_mode)(cgm_read_t*);
    DeviceViewPortMap(*get_device_view_port_map)(cgm_read_t*);
    DeviceViewPortMapH(*get_device_view_port_map_h)(cgm_read_t*);
    DeviceViewPortMapV(*get_device_view_port_map_v)(cgm_read_t*);
    ClipIndicator(*get_clip_indicator)(cgm_read_t*);
    ColorSelectionMode(*get_color_selection_mode)(cgm_read_t*);
    //ColorModel
    close_type_t(*get_close_type)(cgm_read_t*);
    void(*get_char_set_type)(cgm_read_t*);
    CharacterCodingA(*get_character_coding_a)(cgm_read_t*);
    TextPrecision(*get_text_precision)(cgm_read_t*);
    TextPath(*get_text_path)(cgm_read_t*);
    TextAlignmentH(*get_text_alignment_h)(cgm_read_t*);
    TextAlignmentV(*get_text_alignment_v)(cgm_read_t*);
    UnderlineMode(*get_underline_mode)(cgm_read_t*);
    FinalFlag(*get_final_flag)(cgm_read_t*);
    SpecMode(*get_spec_mode)(cgm_read_t*);
    EdgeVisibility(*get_edge_visibility)(cgm_read_t*);
    edge_flag_t(*get_edge_flag)(cgm_read_t*);
    MarkerType(*get_marker_type)(cgm_read_t*);
    Transparency(*get_transparency)(cgm_read_t*);
    interior_style_t(*get_interior_style)(cgm_read_t*);
    int(*get_asf_mask)(cgm_read_t*);
    asf_value_t(*get_asf_value)(cgm_read_t*);

}cgm_read_interface_t;

struct _cgm_read_t{
    const uint8_t *data;
    const uint8_t *end;
    const cgm_read_interface_t *vt;

    int                 int_precision;            /* 1, 2, 4 Bytes */
    int                 index_precision;          /*   "  "  "  */
    real_type_t         real_type;
    int                 real_size;                /*  4 or 8 bytes  */
    int                 color_precision;          /*   "  "  "  */
    int                 color_index_precision;    /*   "  "  "  */

    vdc_type_t          vdc_type;
    int                 vdc_int_precision;
    real_type_t         vdc_real_type;
    int                 vdc_real_size;

    DeviceViewPortMode  eDeviceViewPortMode;
};

cgm_read_t *cgm_read_init(cgm_read_t*);
void cgm_read_set(cgm_read_t*, const uint8_t*, size_t);
#define cgm_read_bytes(r) ((r)->end-(r)->data)

int r_get_int_x(cgm_read_t*, int precicion);
unsigned int r_get_uint_x(cgm_read_t*, int precicion);

double r_get_real_x(cgm_read_t*, real_type_t, int size);
uint32_t r_get_color(cgm_read_t*, const CGMElements* e, int direct);

#define cgm_read_eoe(r) r->vt->eoe(r)

#define r_get_color_channel(r) r->vt->get_color_channel(r)
#define r_get_color_index(r) r->vt->get_color_index(r)
#define r_get_int_precision_bytes(r) r->vt->get_int_precision_bytes(r)

#define r_get_vdc_type(r) r->vt->get_vdc_type(r)
#define r_get_scaling_mode(r) r->vt->get_scaling_mode(r)
#define r_get_device_view_port_mode(r) r->vt->get_device_view_port_mode(r)
#define r_get_device_view_port_map(r) r->vt->get_device_view_port_map(r)
#define r_get_device_view_port_map_h(r) r->vt->get_device_view_port_map_h(r)
#define r_get_device_view_port_map_v(r) r->vt->get_device_view_port_map_v(r)
#define r_get_clip_indicator(r) r->vt->get_clip_indicator(r)
#define r_get_color_selection_mode(r) r->vt->get_color_selection_mode(r)
#define r_get_close_type(r) r->vt->get_close_type(r)
#define r_get_char_set_type(r) r->vt->get_char_set_type(r)
#define r_get_character_coding_a(r) r->vt->get_character_coding_a(r)
#define r_get_text_precision(r) r->vt->get_text_precision(r)
#define r_get_text_path(r) r->vt->get_text_path(r)
#define r_get_text_alignment_h(r) r->vt->get_text_alignment_h(r)
#define r_get_text_alignment_v(r) r->vt->get_text_alignment_v(r)
#define r_get_interior_style(r) r->vt->get_interior_style(r)
#define r_get_underline_mode(r) r->vt->get_underline_mode(r)
#define r_get_final_flag(r) r->vt->get_final_flag(r)
#define r_get_line_type(r) r->vt->get_line_type(r)
#define r_get_spec_mode(r) r->vt->get_spec_mode(r)
#define r_get_line_cap_type(r) r->vt->get_line_cap_type(r)
#define r_get_line_join_type(r) r->vt->get_line_join_type(r)
#define r_get_edge_type(r) r->vt->get_edge_type(r)
#define r_get_edge_visibility(r) r->vt->get_edge_visibility(r)
#define r_get_edge_flag(r) r->vt->get_edge_flag(r)
#define r_get_marker_type(r) r->vt->get_marker_type(r)
#define r_get_transparency(r) r->vt->get_transparency(r)

#define r_get_xpos(r) matrix_apply_x(&(r)->vdc_matrix, r_get_VDC(r))
#define r_get_ypos(r) matrix_apply_y(&(r)->vdc_matrix, r_get_VDC(r))
void r_get_rect(cgm_read_t*, rect_t*);

/* Basic types */

#define r_get_I(r) r->vt->get_I(r)
#define r_get_IX(r) r->vt->get_IX(r)
#define r_get_R(r) r->vt->get_R(r)
#define r_get_S(r,a) r->vt->get_S(r,a)
#define r_get_VDC(r) r->vt->get_vdc(r)
void   r_get_DIRECTCOLR(cgm_read_t*, uint8_t*);
void   r_get_K(cgm_read_t*, uint8_t*);
void   r_get_P(cgm_read_t*, loa_point_t*);
double r_get_V(cgm_read_t*, SpecMode);
//void r_get_N(cgm_read_t*);
double r_get_VC(cgm_read_t*);
double r_get_VP(cgm_read_t*, loa_point_t*);
//void r_get_TM(cgm_read_t*);

/* Enumerated types */

#define r_get_device_view_port_map_h(r) r->vt->get_device_view_port_map_h(r)
#define r_get_device_view_port_map_v(r) r->vt->get_device_view_port_map_v(r)

#define r_get_asf_mask(r) r->vt->get_asf_mask(r)
#define r_get_asf_value(r) r->vt->get_asf_value(r)

#define cgm_get_byte(nSource, nPrecision) (((nSource)>>(((nPrecision)-1)<<3))&0xff)

#endif /* cgmread_h */
