/*
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <stdlib.h>
#include <string.h>

#include "ni-ehb.h"

void
ni_ehb_expand_palette(nile_t *ni)
{
    int allocation;
    int offset;
    int i;
    if(!ni->palette || !ni->colors)
        return;
    if(!ni->depth){
        ni->depth = 1;
        while(1<<ni->depth < ni->colors)
            ++ni->depth;
    }
    offset = 1<<ni->depth;
    allocation = (offset + ni->colors) * ni->bpc;
    if(ni->palette_allocation){
        if(ni->palette_allocation < allocation){
            ni->palette_allocation = allocation;
            ni->palette = realloc(ni->palette, ni->palette_allocation);
        }
    }else{
        uint8_t *old_palette = ni->palette;
        ni->palette_allocation = allocation;
        ni->palette = malloc(ni->palette_allocation);
        memcpy(ni->palette, old_palette, ni->colors * ni->bpc);
    }
    for(i=0;i<ni->colors;++i){
        ni->palette[(i+offset) * ni->bpc + 0] = (ni->palette[i * ni->bpc + 0]>>1) & 0x77;
        ni->palette[(i+offset) * ni->bpc + 1] = (ni->palette[i * ni->bpc + 1]>>1) & 0x77;
        ni->palette[(i+offset) * ni->bpc + 2] = (ni->palette[i * ni->bpc + 2]>>1) & 0x77;
    }
    ni->colors += offset;
    ++ni->depth;
}
