/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
#include <abydos-plugin.h>
#include <nil-cairo.h>
#include <stdlib.h>
#include <string.h>


struct _abydos_plugin_handle_t{
    abydos_plugin_info_t *info;
    cairo_surface_t *surface;
};


static const char **
_d256_get_mime_types()
{
    static const char *mime_types[] = {
        "image/x-draw256-vga",
        NULL
    };
    return mime_types;
}


static abydos_plugin_handle_t *
_d256_new(const char *mime_type, abydos_plugin_info_t *info)
{
    abydos_plugin_handle_t *h = (abydos_plugin_handle_t*)malloc(sizeof(abydos_plugin_handle_t));
    info->threadsafe = 1;
    h->info = info;
    h->surface = NULL;
    return h;
}


static void
_d256_free(abydos_plugin_handle_t *h)
{
    if(h->surface)
        cairo_surface_destroy(h->surface);
    free(h);
}


#define get16(a) le16toh(*(uint16_t*)(a))


static int
_d256_create_from_data(abydos_plugin_handle_t *h,const char *data, size_t len)
{
    if(len < 772)
        return -1;

    h->info->width = get16(data+768);
    h->info->height = get16(data+770);

    if(len < 772 + h->info->width * h->info->height)
        return -1;

    h->surface = nil_cairo_surface_create_from_indexed((uint8_t*)data + 772,
                                                       h->info->width,
                                                       h->info->height,
                                                       -1,
                                                       (uint8_t*)data,
                                                       256,
                                                       3,
                                                       NILE_FORMAT_r8g8b8,
                                                       0);

    if(h->info->width <= 640 && h->info->height <= 400)
        h->info->pixel_ratio = 5.0/6;

    return 0;
}


static cairo_surface_t *
_d256_get_image_surface(abydos_plugin_handle_t *h,
                        int page,
                        cairo_bool_t *layer_visibility,
                        int variant,
                        double time,
                        int frame)
{
    return cairo_surface_reference(h->surface);
}


const abydos_plugin_interface_t *
Abydos0()
{
    static const abydos_plugin_interface_t p = {
        .version = ABYDOS_PLUGIN_INTERFACE_VERSION,
        .get_mime_types = _d256_get_mime_types,
        .new = _d256_new,
        .free = _d256_free,
        .create_from_data = _d256_create_from_data,
        .get_image_surface = _d256_get_image_surface
    };
    return &p;
}
