/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef loa_canvas_h
#define loa_canvas_h

#include "loa-abstract.h"

typedef enum{
    LOA_CANVAS_GROUP,
    LOA_CANVAS_ABSTRACT
}loa_canvas_type_t;

typedef struct _loa_canvas_part_t loa_canvas_part_t;
struct _loa_canvas_part_t{
    loa_abstract_t *abstract;
    double x;
    double y;
    double xscale;
    double yscale;
    int flags;
    loa_canvas_part_t *next;
};

typedef struct{
    int refcount;
    loa_canvas_type_t type;
    union{
        struct{
            loa_canvas_part_t *first;
            loa_canvas_part_t *last;
        }group;
        loa_abstract_t *abstract;
    };
}loa_canvas_t;

loa_canvas_t *loa_canvas_create(loa_canvas_type_t);
loa_canvas_t *loa_canvas_reference(loa_canvas_t*);
void loa_canvas_destroy(loa_canvas_t*, loa_abstract_free_func_t, void *userdata);
void loa_canvas_flatten(loa_canvas_t*, const loa_abstract_interface_t*, void *userdata);
loa_canvas_part_t *loa_canvas_add_part(loa_canvas_t*);
loa_canvas_t *loa_canvas_copy(loa_canvas_t *src, int deep, const loa_abstract_interface_t*, void *userdata);
void loa_canvas_scale(loa_canvas_t*, double xscale, double yscale);
void loa_canvas_paste(loa_canvas_t *dst, double x, double y, double xscale, double yscale, int flags, nil_clip_t*, loa_canvas_t *src, const loa_abstract_interface_t*, void *userdata);

#endif /* loa_canvas_h */
