/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * This library is free software; you can redistribute it and/or modify it
 * under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at
 * your option) any later version.
 *
 * This library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, see <http:/www.gnu.org/licenses/>
 */
extern "C"{
#include <abydos.h>
#include <debug.h>
}
#include "revenge-cairo.h"


static void
_list_properties(const RVNGPropertyList &props)
{
    RVNGPropertyList::Iter iter(props);
    const RVNGProperty *prop;
    while((prop = iter())){
        DEBUG_p("%s", iter.key());
        if(!iter.next())
            break;
    }
}


void
RVNGCairoDrawingGenerator::drawGraphicObject(const RVNGPropertyList &props)
{
    DEBUG_p("begin");
    _list_properties(props);
    auto mime_type = props["librevenge:mime-type"];
    auto base64 = props["office:binary-data"];
    if(!mime_type || !base64)
        return;
    abydos_t *ar = abydos_create(mime_type->getStr().cstr());
    if(!ar)
        return;
    RVNGBinaryData bin(base64->getStr());
    if(0 == abydos_from_data(ar, (const char*)bin.getDataBuffer(), bin.size())){
        if(width && height)
            abydos_get_size(ar, width, height);
        cairo_t *cr = cairo_create(surface);
        abydos_render(ar, cr);
        cairo_destroy(cr);
    }
    abydos_destroy(ar);
}
