/*
 * Copyright (C) 2017  Teemu Suutari
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 */
#ifndef dec_h
#define dec_h

#include <cstddef>
#include <cstdint>
#include <memory>
#include <string>

#include "buffer.h"
#include "common.h"
#include "streamo.h"


struct Decompressor {
    // just a base class to easily catch all the errors
    class Error : public std::exception {};
    class InvalidFormatError : public Error {};
    class DecompressionError : public Error {};
    class VerificationError : public Error {};
};


class XPKDecompressor {
public:
    class State {
    public:
        State(const State&)=delete;
        State& operator=(const State&)=delete;
        State()=default;
        virtual ~State(){};
    };

    XPKDecompressor(const Buffer &packedData) : _packedData(packedData) {}
    XPKDecompressor(const XPKDecompressor&)=delete;
    XPKDecompressor& operator=(const XPKDecompressor&)=delete;
    virtual ~XPKDecompressor(){};

    // Actual decompression
    virtual void decompressImpl(Buffer &rawData, const Buffer &previousData, bool verify)=0;

protected:
    const Buffer &_packedData;
};

#endif /* dec_h */
