/*
 * Copyright (C) 2017  Teemu Suutari
 * Copyright (C) 2020  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * * Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer.
 *
 * * Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 */
#ifndef SXSCDECOMPRESSOR_HPP
#define SXSCDECOMPRESSOR_HPP

#include <cstdint>

#include "dec.h"
#include "streami.h"
#include "dec_range.h"

class SXSCDecompressor : public XPKDecompressor
{
public:
	SXSCDecompressor(const Buffer &packedData,bool isHSC);
	virtual ~SXSCDecompressor(){}

	virtual void decompressImpl(Buffer &rawData,const Buffer &previousData,bool verify) override final;

private:
	class SXSCReader : public RangeDecoder::BitReader
	{
	public:
		SXSCReader(ForwardInputStream &stream);
		virtual ~SXSCReader();

		virtual uint32_t readBit() override final;

	private:
		MSBBitReader<ForwardInputStream>	_reader;
	};

	void decompressASC(Buffer &rawData,ForwardInputStream &inputStream);
	void decompressHSC(Buffer &rawData,ForwardInputStream &inputStream);

	bool						_isHSC;
};

#endif
