/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef xpk_h
#define xpk_h

#include <endian.h>

#if __BYTE_ORDER == __BIG_ENDIAN
# define ID(a,b,c,d) ((a<<24)|(b<<16)|(c<<8)|(d<<0))
#elif __BYTE_ORDER == __LITTLE_ENDIAN
# define ID(a,b,c,d) ((a<<0)|(b<<8)|(c<<16)|(d<<24))
#else
# error "Unknown byte order"
#endif

#include "dec.h"

typedef enum{
    XPK_ACCA = ID('A','C','C','A'),
    XPK_ARTM = ID('A','R','T','M'),
    XPK_BLZW = ID('B','L','Z','W'),
    /* bzip2 ? */
    XPK_CBR0 = ID('C','B','R','0'),
    XPK_CBR1 = ID('C','B','R','1'),
    XPK_CRMS = ID('C','R','M','S'),
    XPK_CRM2 = ID('C','R','M','2'),
    XPK_CYB2 = ID('C','Y','B','2'),
    /* deflate ? */
    XPK_DUKE = ID('D','U','K','E'),
    XPK_ELZX = ID('E','L','Z','X'),
    XPK_FAST = ID('F','A','S','T'),
    XPK_FBR2 = ID('F','B','R','2'),
    XPK_FRHT = ID('F','R','H','T'),
    XPK_FRLE = ID('F','R','L','E'),
    XPK_HFMN = ID('H','F','M','N'),
    XPK_HUFF = ID('H','U','F','F'),
    XPK_ILZR = ID('I','L','Z','R'),
    XPK_IMP  = ID('I','M','P','L'),
    XPK_LHLB = ID('L','H','L','B'),
    XPK_LIN1 = ID('L','I','N','1'),
    XPK_LIN2 = ID('L','I','N','2'),
    XPK_LIN3 = ID('L','I','N','3'),
    XPK_LIN4 = ID('L','I','N','4'),
    XPK_LZBS = ID('L','Z','B','S'),
    XPK_LZCB = ID('L','Z','C','B'),
    XPK_LZW2 = ID('L','Z','W','2'),
    XPK_LZW3 = ID('L','Z','W','3'),
    XPK_LZW4 = ID('L','Z','W','4'),
    XPK_LZW5 = ID('L','Z','W','5'),
    XPK_MASH = ID('M','A','S','H'),
    XPK_NONE = ID('N','O','N','E'),
    XPK_NUKE = ID('N','U','K','E'),
    XPK_PWPK = ID('P','W','P','K'),
    XPK_RAKE = ID('R','A','K','E'),
    XPK_RDCN = ID('R','D','C','N'),
    XPK_RLEN = ID('R','L','E','N'),
    XPK_SASC = ID('S','A','S','C'),
    XPK_SDHC = ID('S','D','H','C'),
    XPK_SHR3 = ID('S','H','R','3'),
    XPK_SHRI = ID('S','H','R','I'),
    XPK_SHSC = ID('S','H','S','C'),
    XPK_SLZX = ID('S','L','Z','X'),
    XPK_SLZ3 = ID('S','L','Z','3'),
    XPK_SMPL = ID('S','M','P','L'),
    XPK_SQSH = ID('S','Q','S','H'),
    /* Stone Cracker ? */
    XPK_TDCS = ID('T','D','C','S'),
    XPK_ZENO = ID('Z','E','N','O')
}xpk_compression_t;

XPKDecompressor *xpk_lookup_decompressor(xpk_compression_t comp,const Buffer &buf,std::unique_ptr<XPKDecompressor::State> &state);

#endif /* xpk_h */
