/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef loa_buffer_h
#define loa_buffer_h

#include "loa.h"
#include "loa-array.h"

typedef struct _loa_object_t loa_object_t;
typedef struct _loa_layer_t loa_layer_t;
typedef struct _loa_anim_t loa_anim_t;
typedef struct _loa_keyframe_t loa_keyframe_t;

typedef struct{
    loa_array_t point_array;
    loa_array_t segment_array;
    loa_array_t object_array;
    loa_array_t layer_array;
    loa_array_t anim_array;
    loa_array_t keyframe_array;
}loa_buffer_t;

loa_buffer_t *loa_buffer_init(loa_buffer_t*, int object_size, int layer_size, int anim_size, int keyframe_size);
void loa_buffer_done(loa_buffer_t*);
void loa_buffer_clear(loa_buffer_t*);

#define loa_buffer_get_point(b,i) ((loa_point_t*)loa_array_get(&(b)->point_array,i))
#define loa_buffer_point_index(b,p) loa_array_index(&(b)->point_array,p)
#define loa_buffer_point_count(b) loa_array_length(&(b)->point_array)
#define loa_buffer_add_points(b,c) (loa_point_t*)loa_array_append(&(b)->point_array,c)
#define loa_buffer_insert_points(b,i,c) (loa_point_t*)loa_array_insert(&(b)->point_array,i,c)
#define loa_buffer_delete_points(b,i,c) loa_array_delete(&(b)->point_array,i,c)
#define loa_buffer_copy_points(b) (loa_point_t*)loa_array_copy(&(b)->point_array)

#define loa_buffer_get_segment(b, i) ((loa_segment_t*)loa_array_get(&(b)->segment_array,i))
#define loa_buffer_segment_count(b) loa_array_length(&(b)->segment_array)
#define loa_buffer_add_segment(b) (loa_segment_t*)loa_array_append(&(b)->segment_array,1)
#define loa_buffer_insert_segment(b, i) (loa_segment_t*)loa_array_insert(&(b)->segment_array,i,1)
#define loa_buffer_delete_segments(b, i, c) loa_array_delete(&(b)->segment_array,i,c)
loa_point_t *loa_buffer_push_segment(loa_buffer_t*, loa_path_t*, loa_segment_type_t);
#define loa_buffer_copy_segments(b) (loa_segment_t*)loa_array_copy(&(b)->segment_array)

#define loa_buffer_get_object(b,i) ((loa_object_t*)loa_array_get(&(b)->object_array,i))
#define loa_buffer_object_index(b,p) loa_array_index(&(b)->object_array,p)
#define loa_buffer_object_count(b) loa_array_length(&(b)->object_array)
#define loa_buffer_add_object(b) (loa_object_t*)loa_array_append(&(b)->object_array,1)
#define loa_buffer_delete_object(b,i) loa_array_delete(&(b)->object_array,i,1)
#define loa_buffer_copy_objects(b) (loa_object_t*)loa_array_copy(&(b)->object_array)

#define loa_buffer_get_layer(b, i) (loa_layer_t*)loa_array_get(&(b)->layer_array, i)
#define loa_buffer_layer_count(b) loa_array_length(&(b)->layer_array)
#define loa_buffer_add_layer(b) (loa_layer_t*)loa_array_append(&(b)->layer_array,1)
#define loa_buffer_copy_layers(b) (loa_layer_t*)loa_array_copy(&(b)->layer_array)

#define loa_buffer_get_anim(b, i) (loa_anim_t*)loa_array_get(&(b)->anim_array, i)
#define loa_buffer_add_anim(b) (loa_anim_t*)loa_array_append(&(b)->anim_array,1)
#define loa_buffer_anim_count(b) loa_array_length(&(b)->anim_array)
#define loa_buffer_copy_anims(b) (loa_anim_t*)loa_array_copy(&(b)->anim_array)

#define loa_buffer_get_keyframe(b, i) ((loa_keyframe_t*)loa_array_get(&(b)->keyframe_array,i))
#define loa_buffer_keyframe_count(b) loa_array_length(&(b)->keyframe_array)
#define loa_buffer_add_keyframe(b) (loa_keyframe_t*)loa_array_append(&(b)->keyframe_array,1)
#define loa_buffer_copy_keyframes(b) (loa_keyframe_t*)loa_array_copy(&(b)->keyframe_array)

#endif /* loa_buffer */
