/*
 * Author: Magnus Bergman <magnus@snisurset.net>
 *
 * This file is public domain.
 */
#ifndef nil_cairo_h
#define nil_cairo_h

#include <cairo/cairo.h>
#include <endian.h>

#include "nile.h"

#if __BYTE_ORDER == __BIG_ENDIAN
# define NIL_CAIRO_FORMAT_ARGB32 (NILE_FORMAT_a8r8g8b8|NILE_PREMULTIPLIED)
# define NIL_CAIRO_FORMAT_RGB24  NILE_FORMAT_x8r8g8b8
#else
# define NIL_CAIRO_FORMAT_ARGB32 (NILE_FORMAT_b8g8r8a8|NILE_PREMULTIPLIED)
# define NIL_CAIRO_FORMAT_RGB24  NILE_FORMAT_b8g8r8
#endif
#define NIL_CAIRO_FORMAT_RGB16_565 NILE_FORMAT_r5g6b5
#define NIL_CAIRO_FORMAT_RGB96F    NILE_FORMAT_r32g32b32
#define NIL_CAIRO_FORMAT_RGBA128F  (NILE_FORMAT_r32g32b32a32|NILE_PREMULTIPLIED)

cairo_format_t nil_cairo_format_from_nile(nile_format_t);
nile_format_t nil_cairo_format_to_nile(cairo_format_t);
int nil_cairo_format_to_bpc(cairo_format_t);

int nil_cairo_convert_palette(nile_t*);
cairo_surface_t *nil_cairo_surface_create_for_format(nile_format_t, int width, int height);
nile_t *nil_cairo_from_surface(nile_t*, cairo_surface_t*);
cairo_surface_t *nil_cairo_surface_create_from_bitplanes(uint8_t **src, int width, int height, int stride, int depth, const uint8_t *palette, int colors, int bpc, nile_format_t, int flags);
cairo_surface_t *nil_cairo_surface_create_from_indexed(const uint8_t *src, int width, int height, int stride, const uint8_t *palette, int colors, int bpc, nile_format_t, int flags);
cairo_surface_t *nil_cairo_surface_create_from_direct(const uint8_t *src,int width,int height,int stride, int bpc, nile_format_t format, int flags);
cairo_surface_t *nil_cairo_surface_create(nile_t*, int flags);

#endif /* nil_cairo_h */
