/*
 * Copyright (C) 2021  Magnus Bergman <magnus@snisurset.net>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 */
#include "debug.h"
#include "nil-iff.h"

#define get32(a) be32toh(*(uint32_t*)(a))


int
nil_iff_parse(uint32_t id, int adj_form_len, const char *data, size_t len, nil_iff_parse_func_t func, void *userdata)
{
    const char *end = data + len;
    if(id){
        size_t form_len;
        if(data + 12 > end)
            return -1;
        if(*(uint32_t*)data != IFF_FORM || *(uint32_t*)(data+8) != id)
            return -1;
        form_len = get32(data+4) + adj_form_len -4;
        data += 12;
        if(data + form_len > end){
            DEBUG_p("truncated data");
            return -1;
        }
        end = data + form_len;
    }

    for(;;){
        int res;
        if(data + 8 > end)
            return 0;
        id = *(uint32_t*)data;
        len = get32(data+4);
        data += 8;
        if(data + len > end)
            return -1;
        res = func(userdata, id, data, len);
        if(res)
            return res;
        len += len&1;
        data += len;
    }
}
